"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.getBitmaskTable = void 0;

var getBitmaskTable = function getBitmaskTable(names) {
  var table = {};
  names.forEach(function (name, i) {
    table[name] = Math.pow(2, i);
  });
  return table;
};

exports.getBitmaskTable = getBitmaskTable;
var MAX_SIGNED_31_BIT_INT = 1073741823;

var parseNames = function parseNames(nameStr) {
  return nameStr.split(/\s*,\s*/);
};

var bitsObserver = function bitsObserver(namesObj) {
  var names = Object.keys(namesObj).sort();
  var bitmaskTable = getBitmaskTable(names);

  var calcChangedBits = function calcChangedBits(a, b) {
    var result = names.reduce(function (r, name) {
      return (// eslint-disable-next-line no-bitwise
        a[name] !== b[name] ? r | bitmaskTable[name] : r
      );
    }, 0); // changed on somethings that not in names

    return result === 0 && a !== b ? MAX_SIGNED_31_BIT_INT : result;
  };

  var getObservedBits = function getObservedBits(observe) {
    if (!observe) return undefined; // observe should be string to recognition

    return parseNames(observe).reduce(function (r, name) {
      return (// eslint-disable-next-line no-bitwise
        r | bitmaskTable[name]
      );
    }, 0);
  };

  return {
    bitmaskTable: bitmaskTable,
    calcChangedBits: calcChangedBits,
    getObservedBits: getObservedBits
  };
};

var _default = bitsObserver;
exports.default = _default;